;Filename: FeeWr.s
;Part: ADuC702x.
;Created: 26/05/2004
;Updated: 28/06/2005
;Author: Eckart Hartmann.
;This file is in beta sampling.

#include        <ADuC702x.IAR.equ>

; FeeWr==========Writes to flash.
; C Function prototype: int FeeWr(int iAddr, char* pcData, int iCnt);
;iAddr:{0-0xf7ff}
;pcData:{}
;iCnt:{1-0xf800}
; Description of Function: Writes iCnt bytes of pcData at 
;   iAddr in flash.
; User interface:
;   Set iAddr to the address where data is to be written.
;   Set pcData to the pointer to the data to write.
;   Set iCnt to the number of bytes to write.
;   Call FeeWr(iAddr, &pcData, iCnt).
;   If iAddr > 0xfdff write nothing.
;   Else write pcData to iAddr except in protected pages. 
;   Returns Number of bytes actually written.
; Robustness:
;   Writing can only change bits to low. To change bits 
;    to high an entire page must be erased (see FeeClr).
;   Obviously writing to a wrong address can be disastrous.
; Side effects: Corrupts r1, r2 and r3.

                RSEG CODE
                MODULE FeeWr
                PUBLIC FeeWr
                CODE32

FeeWr:
        stmfd   sp!,{r4,r5,lr}
        mov     lr,#0                   ;lr = iRet = 0; //lr = iRet.
        cmp     r0,#0xf800              ;if(iAddr>0xf800)   //r0 = iAddr.
        bhi     FeeWR                   ; return iRet = 0;
        add     r2,r0,r2                ;iEnd += iAddr; //r2 = iEnd.
        mov     r3,#0xff000000          ;r3 = F_BASE;
        mov     r3,r3,ASR#12
        mov     r4,r3,ASR#16            ;r4 = 0xffffffff; //r4 = iData.
NxtFW:  cmp     r0,r2                   ;while(iAddr<iEnd)
        bhs     FeeWR                   ; {
        cmp     r0,#0xf800              ; if(iAddr>=0xf800) break;
        bcs     FeeWR
        ldrb    r5,[r1],#1              ; r5 = *pcData++|0xffffff00;
        orr     r5,r5,r3,ASR#4
        tst     r0,#1                   ; if(!(iAddr&1))
        andeq   r4,r4,r5                ;  iData &= r5;
        andne   r4,r4,r5,ROR#24         ; else iData &= r5<<8;
;        tst     r0,#1                   ; if(iAddr&1 ||
        bne     WrF
        add     r5,r0,#1                ;    iAddr+1==iEnd)
        cmp     r5,r2
        bne     SkpFW                   ;  {
WrF:    str     r0,[r3,#F_FEEADR]       ;  FEEADR = iAddr.
        str     r4,[r3,#F_FEEDAT]       ;  FEEDAT = iData.
        mov     r5,#2                   ;  FEECON = 0x2.
        str     r5,[r3,#F_FEECON]
        add     lr,lr,#1                ;  iRet++;
        mov     r4,r3,ASR#16            ;  r4 = 0xffffffff;
ChkWDn: ldr     r5,[r3,#F_FEESTA]       ;  while(!FEESTA&1));
        tst     r5,#1
        beq     ChkWDn                  ;  }
SkpFW:  add     r0,r0,#1                ; iAddr++;
        b       NxtFW                   ; }
FeeWR:  mov     r0,lr
        ldmfd   sp!,{r4,r5,lr}
        bx      lr                      ;return

        bne     SkpFC
        str     r0,[r3,#F_FEEADR]       ;   FEEADR = r0;
        mov     r2,#5                   ;   FEECON = 5;
        str     r2,[r3,#F_FEECON]
        add     lr,lr,#1                ;   iPageCnt++;
ChkEDn: ldr     r2,[r3,#F_FEESTA]       ;   while(!FEESTA&1);
        tst     r2,#1
        beq     ChkEDn                  ;  }}
SkpFC:  add     r0,r0,#0x200            ; r0 += 0x200;
        b       NxtFW                   ; }
RetFC:  mov     r0,lr                   ;return iRet;
        ldmfd   sp!,{r4,lr}
        bx      lr                   ;return


                 ENDMOD FeeWr 

                 END
        
